package cz.frantovo.nekurak.dao;

import cz.frantovo.nekurak.dto.Podnik;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Collection;
import java.util.logging.Level;

/**
 * Stará se o načítání a ukládání podniků (hospod) z a do databáze.
 * @author fiki
 */
public class PodnikDAO extends NekurakSuperDAO {

    private enum SQL {

	SELECT_VSECHNY,
	INSERT,
	UPDATE,
	NEXTVAL
    }

    public Collection<Podnik> getPodniky() {
	Connection db = getSpojeni();
	PreparedStatement ps = null;
	ResultSet rs = null;
	try {
	    ps = db.prepareStatement(getSQL(SQL.SELECT_VSECHNY));
	    rs = ps.executeQuery();
	    Collection<Podnik> vysledek = new ArrayList<Podnik>();

	    while (rs.next()) {
		vysledek.add(new Podnik(rs.getInt("id"), rs.getString("nazev")));
	    }

	    return vysledek;
	} catch (Exception e) {
	    log.log(Level.SEVERE, "Chyba při získávání podniků.", e);
	    return null;
	} finally {
	    zavri(db, ps, rs);
	}
    }

    /**
     * Uloží podnik do databáze.
     * @param p Podnik k uložení. Pokud má být vložen nový, má id = 0.
     * @return id vloženého nebo uloženého podniku.
     */
    public int ulozPodnik(Podnik p) {
	Connection db = getSpojeni();
	PreparedStatement psUpdate = null;
	PreparedStatement psId = null;
	ResultSet rsId = null;
	try {
	    if (p.getId() == 0) {
		psId = db.prepareStatement(getSQL(SQL.NEXTVAL));
		rsId = psId.executeQuery();
		rsId.next();
		p.setId(rsId.getInt(1));

		psUpdate = db.prepareStatement(getSQL(SQL.INSERT));
	    } else {
		psUpdate = db.prepareStatement(getSQL(SQL.UPDATE));
	    }

	    psUpdate.setString(1, p.getNazev());
	    psUpdate.setInt(2, p.getId());

	    psUpdate.executeUpdate();

	    db.commit();
	    return p.getId();
	} catch (Exception e) {
	    rollback(db);
	    log.log(Level.SEVERE, "Chyba při ukládání podniku.", e);
	    return -1;
	} finally {
	    zavri(null, psId, rsId);
	    zavri(db, psUpdate, null);
	}
    }
}
